$wnd.rstudio.runAsyncCallback16("function J5s(a){this.a=a}\nfunction _2s(a,b){this.a=a;this.b=b;cdf.call(this)}\nfunction D2s(a,b){a.a=b;X4s(a.p,z2s(a,null),a.a.file_path)}\nfunction F2s(a,b){b.slide_index==0&&iUp(a.p);a.a=b;X4s(a.p,z2s(a,null),a.a.file_path)}\nfunction M2s(a){Zah(x2s.$mc(),pge());rPh(a.c,new mRh,false)}\nfunction A2s(a){var b;b=rdi(new sdi(0,x2s.Wmc()));q2l(a.n,new _2s(a,b))}\nfunction q2l(a,b){K7l(a,Kty,'close_presentation_pane',new uL,b)}\nfunction L2s(a){var b;a.J.r4b();!!a.a&&a.a.active&&(b=caf(Oh(a.p.d).contentWindow),!(!Pte(XWx,b)&&!Pte('javascript:void(0)',b)))&&D2s(a,a.a)}\nfunction F5s(a){var b,c;try{if(a.c<0)return;if(!a.a.a){a.a.a=t6s(a.a.b.a);sOp(a.a,a.a.a)}switch(a.c){case 1:A2s((c=a.a.a,a.b[0],c));break;case 2:F2s(a.a.a,a.b[0]);break;case 3:a.a.a.J.p4b();break;case 4:a.a.a.J.q4b();break;case 5:L2s(a.a.a);break;case 6:M2s((b=a.a.a,a.b[0],b));break;case 7:a.a.a.J.s4b();}}finally{a.c<=0&&a.b[0]!=null&&a.b[0].ln()}}\nvar cYA='Closing Presentation...',dYA='Opening Presentation...';sFd(7924,33,{},_2s);_.oA=function b3s(a){var b;rPh((b=this,a,b).a.c,new mRh,false)};_.nA=function a3s(a){Ici(this.b,a.a?a.a.c:a.d)};var vKc=dse(7924);sFd(8751,1,juA);_.Wmc=function G3s(){return cYA};_.$mc=function L3s(){return dYA};sFd(8749,1,juA);_.Wmc=function Y3s(){return cYA};_.$mc=function b4s(){return dYA};sFd(8750,1,juA);_.Wmc=function o4s(){return 'Fermeture de la pr\\xE9sentation...'};_.$mc=function t4s(){return 'Ouvrir la pr\\xE9sentation...'};sFd(459,1,Kqy);_.jn=function I5s(){F5s((new J5s(this)).a)};sFd(7223,1,oTx,J5s);_.ln=function K5s(){F5s(this.a)};var XKc=dse(7223);lDx(fm)(16);\n//# sourceURL=rstudio-16.js\n")
