
<header id="title-block-header" class="quarto-title-block default$if(quarto-template-params.banner-header-class)$ $quarto-template-params.banner-header-class$$endif$">
  <div class="quarto-title-banner">
    <div class="quarto-title column-body">
      <h1 class="title">$title$</h1>
      $if(subtitle)$
      <p class="subtitle lead">$subtitle$</p>
      $endif$
      $if(description)$
      <div>
        <div class="description">
          $description$
        </div>
      </div>
      $endif$
      $if(categories)$
        $if(quarto-template-params.title-block-categories)$
      <div class="quarto-categories">
        $for(categories)$
        <div class="quarto-category">$categories$</div>
        $endfor$
      </div>
        $endif$
      $endif$
    </div>
  </div>
    
  $title-metadata.html()$
  
  </header>